@@ -15,16 +15,10 @@ from . import render_latex
 from .args_parser import create_parser
 from .const import BED_AND_BREAKFAST_DAYS, CAPITAL_GAIN_ALLOWANCES, INTERNAL_START_DATE
 from .currency_converter import CurrencyConverter
-from .dates import (
-    date_from_index,
-    date_to_index,
-    get_tax_year_end,
-    get_tax_year_start,
-    is_date,
-)
+from .dates import get_tax_year_end, get_tax_year_start, is_date
 from .exceptions import (
     AmountMissingError,
-    CalculatedAmountDiscrepancy,
+    CalculatedAmountDiscrepancyError,
     CalculationError,
     InvalidTransactionError,
     PriceMissingError,
@@ -52,6 +46,14 @@ from .util import round_decimal
 LOGGER = logging.getLogger(__name__)
 
 
+def get_amount_or_fail(transaction: BrokerTransaction) -> Decimal:
+    """Return the transaction amount or throw an error."""
+    amount = transaction.amount
+    if amount is None:
+        raise AmountMissingError(transaction)
+    return amount
+
+
 class CapitalGainsCalculator:
     """Main calculator class."""
 
@@ -81,6 +83,7 @@ class CapitalGainsCalculator:
         """Add new acquisition to the given list."""
         symbol = transaction.symbol
         quantity = transaction.quantity
+        price = transaction.price
         if symbol is None:
             raise SymbolMissingError(transaction)
         if quantity is None or quantity <= 0:
@@ -90,32 +93,25 @@ class CapitalGainsCalculator:
             portfolio[symbol] += quantity
         else:
             portfolio[symbol] = quantity
+
         # Add to acquisition_list to apply same day rule
         if transaction.action in [ActionType.STOCK_ACTIVITY, ActionType.SPIN_OFF]:
-            stock_price_gbp = None
-
-            if transaction.price is not None and transaction.currency is not None:
-                stock_price_gbp = self.converter.to_gbp(
-                    transaction.price, transaction.currency, transaction.date
-                )
-            else:
-                stock_price_gbp = self.initial_prices.get(transaction.date, symbol)
-
-            amount = quantity * stock_price_gbp
+            if price is None:
+                price = self.initial_prices.get(transaction.date, symbol)
+            amount = round_decimal(quantity * price, 2)
         else:
-            if transaction.amount is None:
-                raise AmountMissingError(transaction)
-            if transaction.price is None:
+            if price is None:
                 raise PriceMissingError(transaction)
-            calculated_amount = round_decimal(
-                quantity * transaction.price + transaction.fees, 2
-            )
-            if transaction.amount != -calculated_amount:
-                raise CalculatedAmountDiscrepancy(transaction, -calculated_amount)
-            amount = -transaction.amount
+
+            amount = get_amount_or_fail(transaction)
+            calculated_amount = round_decimal(quantity * price + transaction.fees, 2)
+            if amount != -calculated_amount:
+                raise CalculatedAmountDiscrepancyError(transaction, -calculated_amount)
+            amount = -amount
+
         add_to_list(
             acquisition_list,
-            date_to_index(transaction.date),
+            transaction.date,
             symbol,
             quantity,
             self.converter.to_gbp_for(amount, transaction),
@@ -149,19 +145,18 @@ class CapitalGainsCalculator:
         if portfolio[symbol] == 0:
             del portfolio[symbol]
         # Add to disposal_list to apply same day rule
-        if transaction.amount is None:
-            raise AmountMissingError(transaction)
-        if transaction.price is None:
+
+        amount = get_amount_or_fail(transaction)
+        price = transaction.price
+
+        if price is None:
             raise PriceMissingError(transaction)
-        amount = transaction.amount
-        calculated_amount = round_decimal(
-            quantity * transaction.price - transaction.fees, 2
-        )
+        calculated_amount = round_decimal(quantity * price - transaction.fees, 2)
         if amount != calculated_amount:
-            raise CalculatedAmountDiscrepancy(transaction, calculated_amount)
+            raise CalculatedAmountDiscrepancyError(transaction, calculated_amount)
         add_to_list(
             disposal_list,
-            date_to_index(transaction.date),
+            transaction.date,
             symbol,
             quantity,
             self.converter.to_gbp_for(amount, transaction),
@@ -186,35 +181,27 @@ class CapitalGainsCalculator:
         for i, transaction in enumerate(transactions):
             new_balance = balance[(transaction.broker, transaction.currency)]
             if transaction.action is ActionType.TRANSFER:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
+                new_balance += get_amount_or_fail(transaction)
             elif transaction.action is ActionType.BUY:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
+                new_balance += get_amount_or_fail(transaction)
                 self.add_acquisition(portfolio, acquisition_list, transaction)
             elif transaction.action is ActionType.SELL:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
+                amount = get_amount_or_fail(transaction)
+                new_balance += amount
                 self.add_disposal(portfolio, disposal_list, transaction)
                 if self.date_in_tax_year(transaction.date):
-                    total_sells += self.converter.to_gbp_for(
-                        transaction.amount, transaction
-                    )
+                    total_sells += self.converter.to_gbp_for(amount, transaction)
             elif transaction.action is ActionType.FEE:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
-                transaction.fees = -transaction.amount
+                amount = get_amount_or_fail(transaction)
+                new_balance += amount
+                transaction.fees = -amount
                 transaction.quantity = Decimal(0)
                 gbp_fees = self.converter.to_gbp_for(transaction.fees, transaction)
                 if transaction.symbol is None:
                     raise SymbolMissingError(transaction)
                 add_to_list(
                     acquisition_list,
-                    date_to_index(transaction.date),
+                    transaction.date,
                     transaction.symbol,
                     transaction.quantity,
                     gbp_fees,
@@ -223,29 +210,20 @@ class CapitalGainsCalculator:
             elif transaction.action in [ActionType.STOCK_ACTIVITY, ActionType.SPIN_OFF]:
                 self.add_acquisition(portfolio, acquisition_list, transaction)
             elif transaction.action in [ActionType.DIVIDEND, ActionType.CAPITAL_GAIN]:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
+                amount = get_amount_or_fail(transaction)
+                new_balance += amount
                 if self.date_in_tax_year(transaction.date):
-                    dividends += self.converter.to_gbp_for(
-                        transaction.amount, transaction
-                    )
+                    dividends += self.converter.to_gbp_for(amount, transaction)
             elif transaction.action in [ActionType.TAX, ActionType.ADJUSTMENT]:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
+                amount = get_amount_or_fail(transaction)
+                new_balance += amount
                 if self.date_in_tax_year(transaction.date):
-                    dividends_tax += self.converter.to_gbp_for(
-                        transaction.amount, transaction
-                    )
+                    dividends_tax += self.converter.to_gbp_for(amount, transaction)
             elif transaction.action is ActionType.INTEREST:
-                if transaction.amount is None:
-                    raise AmountMissingError(transaction)
-                new_balance += transaction.amount
+                amount = get_amount_or_fail(transaction)
+                new_balance += amount
                 if self.date_in_tax_year(transaction.date):
-                    interest += self.converter.to_gbp_for(
-                        transaction.amount, transaction
-                    )
+                    interest += self.converter.to_gbp_for(amount, transaction)
             else:
                 raise InvalidTransactionError(
                     transaction, f"Action not processed({transaction.action})"
@@ -277,7 +255,7 @@ class CapitalGainsCalculator:
         bed_and_breakfast_list: HmrcTransactionLog,
         portfolio: dict[str, tuple[Decimal, Decimal]],
         symbol: str,
-        date_index: int,
+        date_index: datetime.date,
     ) -> list[CalculationEntry]:
         """Process single acquisition."""
         acquisition_quantity, acquisition_amount, acquisition_fees = acquisition_list[
@@ -349,7 +327,7 @@ class CapitalGainsCalculator:
         bed_and_breakfast_list: HmrcTransactionLog,
         portfolio: dict[str, tuple[Decimal, Decimal]],
         symbol: str,
-        date_index: int,
+        date_index: datetime.date,
     ) -> tuple[Decimal, list[CalculationEntry]]:
         """Process single disposal."""
         disposal_quantity, proceeds_amount, disposal_fees = disposal_list[date_index][
@@ -405,7 +383,7 @@ class CapitalGainsCalculator:
         # Bed and breakfast rule next
         if disposal_quantity > 0:
             for i in range(BED_AND_BREAKFAST_DAYS):
-                search_index = date_index + i + 1
+                search_index = date_index + datetime.timedelta(days=i + 1)
                 if has_key(acquisition_list, search_index, symbol):
                     (
                         acquisition_quantity,
@@ -442,8 +420,8 @@ class CapitalGainsCalculator:
                         continue
                     print(
                         f"WARNING: Bed and breakfasting for {symbol}."
-                        f" Disposed on {date_from_index(date_index)}"
-                        f" and acquired again on {date_from_index(search_index)}"
+                        f" Disposed on {date_index}"
+                        f" and acquired again on {search_index}"
                     )
                     available_quantity = min(
                         disposal_quantity,
@@ -530,6 +508,9 @@ class CapitalGainsCalculator:
                     new_pool_cost=current_amount,
                 )
             )
+            disposal_quantity = Decimal(0)
+
+        assert disposal_quantity == 0, f"disposal quantity {disposal_quantity}"
         portfolio[symbol] = (current_quantity, current_amount)
         chargeable_gain = round_decimal(chargeable_gain, 2)
         return chargeable_gain, calculation_entries
@@ -540,9 +521,9 @@ class CapitalGainsCalculator:
         disposal_list: HmrcTransactionLog,
     ) -> CapitalGainsReport:
         """Calculate capital gain and return generated report."""
-        begin_index = date_to_index(INTERNAL_START_DATE)
-        tax_year_start_index = date_to_index(self.tax_year_start_date)
-        end_index = date_to_index(self.tax_year_end_date)
+        begin_index = INTERNAL_START_DATE
+        tax_year_start_index = self.tax_year_start_date
+        end_index = self.tax_year_end_date
         disposal_count = 0
         disposal_proceeds = Decimal(0)
         allowable_costs = Decimal(0)
@@ -551,7 +532,10 @@ class CapitalGainsCalculator:
         bed_and_breakfast_list: HmrcTransactionLog = {}
         portfolio: dict[str, tuple[Decimal, Decimal]] = {}
         calculation_log: CalculationLog = {}
-        for date_index in range(begin_index, end_index + 1):
+        for date_index in (
+            begin_index + datetime.timedelta(days=x)
+            for x in range(0, (end_index - begin_index).days)
+        ):
             if date_index in acquisition_list:
                 for symbol in acquisition_list[date_index]:
                     calculation_entries = self.process_acquisition(
@@ -592,7 +576,7 @@ class CapitalGainsCalculator:
                         transaction_quantity = disposal_list[date_index][symbol][0]
                         LOGGER.debug(
                             "DISPOSAL on %s of %s, quantity %d, capital gain $%s",
-                            date_from_index(date_index),
+                            date_index,
                             symbol,
                             transaction_quantity,
                             round_decimal(transaction_capital_gain, 2),
@@ -656,7 +640,7 @@ def main() -> int:
 
     # Read data from input files
     broker_transactions = read_broker_transactions(
-        args.schwab, args.trading212, args.mssb
+        args.schwab, args.schwab_award, args.trading212, args.mssb
     )
     converter = CurrencyConverter(read_gbp_prices_history(args.gbp_history))
     initial_prices = InitialPrices(read_initial_prices(args.initial_prices))
